uniform sampler2D tex;
uniform vec4 poly_color;
uniform vec4 tint_color;
uniform vec4 fade_color;
uniform float lighting;
uniform float fade_start;
uniform float fade_end;
uniform float leveltime;
const float freq = 0.007;
const float amp = 0.007;
const float speed = 3.0;
const float pi = 22 / 7;

float R_DoomColormap(float light, float z)
{
	float lightnum = clamp(light / 16.9, 0.0, 16.0);
	float lightz = clamp(z / 18.0, 0.0, 127.0);
	float startmap = (15.0 - lightnum) * 4.0;
	float scale = 160.0 / (lightz + 1.0);
	return startmap - scale * 0.9;
}

float R_DoomLightingEquation(float light)
{
	float z = gl_FragCoord.z / gl_FragCoord.w;
	float colormap = floor(R_DoomColormap(light, z)) + 0.45;
	return clamp(colormap, 0.0, 31.0) / 32.0;
}

vec4 R_SoftwareTint(vec4 base_color, vec4 final_color) {
	if (tint_color.a > 0.0) {
		float color_bright = sqrt((base_color.r * base_color.r) + (base_color.g * base_color.g) + (base_color.b * base_color.b));
		float strength = sqrt(7.0 * tint_color.a);
		final_color.r = clamp((color_bright * (tint_color.r * strength)) + (base_color.r * (1.0 - strength)), 0.0, 1.0);
		final_color.g = clamp((color_bright * (tint_color.g * strength)) + (base_color.g * (1.0 - strength)), 0.0, 1.0);
		final_color.b = clamp((color_bright * (tint_color.b * strength)) + (base_color.b * (1.0 - strength)), 0.0, 1.0);
	}
	if (tint_color.r > 0.5) {
		float color_bright = sqrt((base_color.r * base_color.r) + (base_color.g * base_color.g) + (base_color.b * base_color.b));
		float strength = sqrt(5.0 * tint_color.a);
		final_color.r = clamp((color_bright * (tint_color.r * strength)) + (base_color.r * (1.0 - strength)), 0.0, 0.9);
		final_color.g = clamp((color_bright * (tint_color.g * strength)) + (base_color.g * (1.0 - strength)), 0.0, 0.8);
		final_color.b = clamp((color_bright * (tint_color.b * strength)) + (base_color.b * (1.0 - strength)), 0.0, 1.0);
	}

	return final_color;
}

vec4 R_SoftwareFade(vec4 final_color) {
	float darkness = R_DoomLightingEquation(lighting);
	if (fade_start != 0.0 || fade_end != 31.0) {
		float fs = fade_start / 31.0;
		float fe = fade_end / 31.0;
		float fd = fe - fs;
		darkness = clamp((darkness - fs) * (1.0 / fd), 0.0, 1.0);
	}
	final_color = mix(final_color, fade_color, darkness);

	return final_color;
}

void base(void) {
	if (tint_color.b > 0.500 && fade_color.b < 0.300) {
		float z = (gl_FragCoord.y / gl_FragCoord.z) / 0.5;
		float a = -pi * (z * freq) + (leveltime * speed);
		float sdistort = cos(a) * amp;
		float cdistort = sin(a) * amp;
		vec4 texel = texture2D(tex, vec2(gl_TexCoord[0].s - sdistort, gl_TexCoord[0].t - cdistort));
		vec4 base_color = texel * poly_color;
		vec4 final_color = base_color;
		final_color = R_SoftwareTint(base_color, final_color);
		final_color = R_SoftwareFade(final_color);
		final_color.a = texel.a * poly_color.a;
		gl_FragColor = final_color;
	}
	else if (tint_color.r > 0.3 && tint_color.b < 0.3 && fade_color.b < 0.300 && tint_color.g < 0.200) {
		float z = (gl_FragCoord.y / gl_FragCoord.z) / 0.5;
		float a = -pi * (z * freq) + (leveltime * speed);
		float sdistort = cos(a) * amp;
		float cdistort = sin(a) * amp;
		vec4 texel = texture2D(tex, vec2(gl_TexCoord[0].s - sdistort, gl_TexCoord[0].t - cdistort));
		vec4 base_color = texel * poly_color;
		vec4 final_color = base_color;
		final_color = R_SoftwareTint(base_color, final_color);
		final_color = R_SoftwareFade(final_color);
		final_color.a = texel.a * poly_color.a;
		gl_FragColor = final_color;
	}
	else {
		vec4 texel = texture2D(tex, gl_TexCoord[0].st);
		vec4 base_color = texel * poly_color;
		vec4 final_color = base_color;
		final_color = R_SoftwareTint(base_color, final_color);
		final_color = R_SoftwareFade(final_color);
		final_color.a = texel.a * poly_color.a;
		gl_FragColor = final_color;
	}
}

// Appended to the end of all GLSL files
float posterize(float val, int levels) {
	int posterized = int(val * levels + 0.5);
	val = float(posterized) / levels;

	return val;
}

void clamp(void) {
	gl_FragColor.r = posterize(gl_FragColor.r, 11);
	gl_FragColor.g = posterize(gl_FragColor.g, 11);
	gl_FragColor.b = posterize(gl_FragColor.b, 11);
	gl_FragColor.a = posterize(gl_FragColor.a, 7);
}

void main(void) {
	base();
	clamp();
}
