uniform sampler2D tex;
uniform vec4 poly_color;
uniform vec4 tint_color;
uniform vec4 fade_color;
uniform float lighting;
uniform float fade_start;
uniform float fade_end;
uniform float leveltime;
const float freq = 0.025;
const float amp = 0.025;
const float speed = 2.0;
const float pi = 3.14159;

float R_DoomColormap(float light, float z)
{
	float lightnum = clamp(light / 19.0, 0.0, 15.0);
	float lightz = clamp(z / 16.0, 0.0, 127.0);
	float startmap = (15.0 - lightnum) * 4.0;
	float scale = 160.0 / (lightz + 1.0);
	return startmap - scale * 1;
}

float R_DoomLightingEquation(float light)
{
	float z = gl_FragCoord.z / gl_FragCoord.w;
	float colormap = floor(R_DoomColormap(light, z)) + 0.3;
	return clamp(colormap, 0.0, 31.0) / 32.0;
}

vec4 R_SoftwareTint(vec4 base_color, vec4 final_color) {
	if (tint_color.a > 0.0) {
		float color_bright = sqrt((base_color.r * base_color.r) + (base_color.g * base_color.g) + (base_color.b * base_color.b));
		float strength = sqrt(9.0 * tint_color.a);
		final_color.r = clamp((color_bright * (tint_color.r * strength)) + (base_color.r * (1.2 - strength)), 0.0, 1.0);
		final_color.g = clamp((color_bright * (tint_color.g * strength)) + (base_color.g * (1.2 - strength)), 0.0, 1.0);
		final_color.b = clamp((color_bright * (tint_color.b * strength)) + (base_color.b * (1.0 - strength)), 0.0, 1.0);
	}

	return final_color;
}

vec4 R_SoftwareFade(vec4 final_color) {
	float darkness = R_DoomLightingEquation(lighting);
	if (fade_start != 0.0 || fade_end != 40.0) {
		float fs = fade_start / 28.0;
		float fe = fade_end / 28.0;
		float fd = fe - fs;
		darkness = clamp((darkness - fs) * (1.0 / fd), 0.0, 1.0);
	}
	final_color = mix(final_color, fade_color, darkness);
	
	return final_color;
}

void base(void) {
	float z = (gl_FragCoord.z / gl_FragCoord.w) / 2.0;
	float a = -pi * (z * freq) + (leveltime * speed);
	float sdistort = sin(a) * amp;
	float cdistort = cos(a) * amp;
	vec4 texel = texture2D(tex, vec2(gl_TexCoord[0].s - sdistort, gl_TexCoord[0].t - cdistort));
	vec4 base_color = texel * poly_color;
	vec4 final_color = base_color;
	final_color = R_SoftwareTint(base_color, final_color);
	final_color = R_SoftwareFade(final_color);
	final_color.a = texel.a * poly_color.a;
	gl_FragColor = final_color;
}

// Appended to the end of all GLSL files
float posterize(float val, int levels) {
	int posterized = int(val * levels + 0.5);
	val = float(posterized) / levels;

	return val;
}

void clamp(void) {
	if (tint_color.r > 0.9) {
		gl_FragColor.r = posterize((gl_FragColor.r - 0.030), 17);
		gl_FragColor.g = posterize(gl_FragColor.g, 14);
		gl_FragColor.b = posterize(gl_FragColor.b, 17);
		gl_FragColor.a = posterize(gl_FragColor.a, 10);
	}
	else {
		gl_FragColor.r = posterize(gl_FragColor.r, 10);
		gl_FragColor.g = posterize(gl_FragColor.g, 10) - 0.030;
		gl_FragColor.b = posterize(gl_FragColor.b, 10);
		gl_FragColor.a = posterize(gl_FragColor.a, 10);
	}
}

void main(void) {
	base();
	clamp();
}